#############################################Packages required########################################
library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)



sir_s_.model.seasonality.socdis.vax.heterogeneity<-function (t,x,parameters) {
  
  SP=x[1]
  IP1=x[2]
  IP2=x[3]
  R1=x[4]
  R2=x[5]
  SS1=x[6]
  SS2=x[7]
  IS1=x[8]
  IS2=x[9]
  V=x[10]
  
  N=parameters[["N"]]
  mu=parameters[["mu"]]
  sigmaP=parameters[["sigmaP"]]
  sigmaS=parameters[["sigmaS"]]
  xsevP=parameters[["xsevP"]]
  xsevS=parameters[["xsevS"]]
  delta1=parameters[["delta1"]]
  delta2=parameters[["delta2"]]
  gamma1=parameters[["gamma1"]]
  gamma2=parameters[["gamma2"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon1=parameters[["epsilon1"]]
  epsilon2=parameters[["epsilon2"]]
  beta=R0.list[t]*gamma1
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  #Vaccination
  nu=parameters[["nu"]]
  delta_vax=parameters[["delta_vax"]]
  tvax=parameters[["tvax"]]
  
  if((t > tred1) & (t < tred2)){
    beta = beta * R0red
  }
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Equations
  dSPdt=mu*N-(beta*SP*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)-mu*SP-svax*nu*SP
  dIP1dt=(beta*SP*(1-xsevP)*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)-(gamma1+mu)*IP1
  dIP2dt=(beta*SP*xsevP*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)-(gamma2+mu)*IP2
  dR1dt=(1-xsevS)*gamma1*IP1+gamma1*IS1+gamma2*(IP2+IS2)-(delta1+mu)*R1
  dR2dt=xsevS*gamma1*IP1-(delta2+mu)*R2
  dSS1dt=delta1*R1+delta_vax*V-mu*SS1-svax*nu*SS1-epsilon1*(beta*SS1*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)
  dSS2dt=delta2*R2-mu*SS2-svax*nu*SS2-epsilon2*(beta*SS2*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)
  dIS1dt=epsilon1*(beta*SS1*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)-(gamma1+mu)*IS1
  dIS2dt=epsilon2*(beta*SS2*(IP1+alpha*IS1+sigmaP*IP2+sigmaS*alpha*IS2)/N)-(gamma2+mu)*IS2
  dVdt=svax*nu*(SP+SS1+SS2)-delta_vax*V-mu*V
  
  dxdt <- c(dSPdt,dIP1dt,dIP2dt,dR1dt,dR2dt,dSS1dt,dSS2dt,dIS1dt,dIS2dt,dVdt)
  list(dxdt)
}


plots.soc.dis.seasonality.vax.heterogeneity=function(alpha,tred1,tred2,tred3,
                                                tred4,delta1,delta2,delta_vax,tvax,nuVector,
                                                primary.burden,secondary.burden,
                                                epsilon1,epsilon2,sigmaP,sigmaS){
  valuesSP=list()
  valuesIP1=list()
  valuesIP2=list()
  valuesR1=list()
  valuesR2=list()
  valuesSS1=list()
  valuesSS2=list()
  valuesIS1=list()
  valuesIS2=list()
  valuesV=list()
  plots=list()
  
  N=1
  mu=1/(50*52)
  gamma1=7/5
  gamma2=gamma1

  R0red=0.6
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  is.col=rgb(1,0.81,0.85)
  v.col=rgb(0.33,0.51,0.21)
  
  xstart=c(SP=(1-I0),IP1=I0*(1-xsevp),IP2=I0*xsevp,R1=0,R2=0,SS1=0,SS2=0, 
           IS1=0,IS2=0,V=0) 
  for  (i in  1:length(nuVector)){
    nu=nuVector[i]
    parameters=list(N=N,mu=mu,gamma1=gamma1,gamma2=gamma2,alpha=alpha,
                    epsilon1=epsilon1,epsilon2=epsilon2,delta1=delta1,delta2=delta2,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tred3=tred3,tred4=tred4,tvax=tvax,nu=nu,
                    delta_vax=delta_vax,sigmaP=sigmaP,sigmaS=sigmaS,
                    xsevP=primary.burden,xsevS=secondary.burden)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax.heterogeneity,
                         y=xstart,times=times,parms=parameters))
    
    valuesIP1[[i]]=out$IP1
    valuesIS1[[i]]=out$IS1
    valuesR1[[i]]=out$R1
    valuesSS1[[i]]=out$SS1
    forAreaPlotIP1=valuesIP1[[i]]
    forAreaPlotIS1=valuesIS1[[i]]
    forAreaPlotR1=valuesR1[[i]]
    forAreaPlotSS1=valuesSS1[[i]]
    valuesIP2[[i]]=out$IP2
    valuesIS2[[i]]=out$IS2
    valuesR2[[i]]=out$R2
    valuesSS2[[i]]=out$SS2    
    forAreaPlotIP2=valuesIP2[[i]]
    forAreaPlotIS2=valuesIS2[[i]]
    forAreaPlotR2=valuesR2[[i]]
    forAreaPlotSS2=valuesSS2[[i]]    
    valuesSP[[i]]=out$SP
    valuesV[[i]]=out$V
    forAreaPlotSP=valuesSP[[i]]
    forAreaPlotV=valuesV[[i]]
    
    burden=forAreaPlotIP2+forAreaPlotIS2
    data.burden=data.frame(xx=times,yy=burden)
    plot.top=ggplot(data=data.burden)+geom_line(aes(x=xx/52,y=yy))+
      ylab("Severe cases")+xlab("Time (years)")+scale_x_continuous(expand=c(0,0))+
      theme_classic((size=14))
    
    data.AP.IP1=data.frame(xx=times,yy=forAreaPlotIP1)
    data.AP.IS1=data.frame(xx=times,yy=forAreaPlotIS1)
    data.AP.R1=data.frame(xx=times,yy=forAreaPlotR1)
    data.AP.SS1=data.frame(xx=times,yy=forAreaPlotSS1)
    data.AP.IP2=data.frame(xx=times,yy=forAreaPlotIP2)
    data.AP.IS2=data.frame(xx=times,yy=forAreaPlotIS2)
    data.AP.R2=data.frame(xx=times,yy=forAreaPlotR2)
    data.AP.SS2=data.frame(xx=times,yy=forAreaPlotSS2)   
    data.AP.SP=data.frame(xx=times,yy=forAreaPlotSP)
    data.AP.V=data.frame(xx=times,yy=forAreaPlotV)
    
    data.AP.IPtotal=data.frame(xx=times,yy=(forAreaPlotIP1+forAreaPlotIP2))
    data.AP.IStotal=data.frame(xx=times,yy=(forAreaPlotIS1+forAreaPlotIS2))
    data.AP.Rtotal=data.frame(xx=times,yy=(forAreaPlotR1+forAreaPlotR2))
    data.AP.SStotal=data.frame(xx=times,yy=(forAreaPlotSS1+forAreaPlotSS2))
    
    data.AP.list=list("primary infection"=data.AP.IPtotal,"secondary infection"=data.AP.IStotal,
                      "fully immune"=data.AP.Rtotal,"partially immune"=data.AP.SStotal,
                      "fully susceptible"=data.AP.SP,
                      "vaccinated"=data.AP.V)
    
    area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
    area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("vaccinated","fully susceptible","fully immune",
                                                                                    "partially immune","secondary infection","primary infection"))
    plot.below=ggplot(area.plots.timecourses,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
      theme_classic((size=14))+
      scale_y_continuous(name="Fraction of population",expand=c(0,0))+
      scale_x_continuous(name="Time (years)",expand=c(0,0))+theme(legend.position="none")+
      scale_fill_manual(name="",values=c(v.col,sp.col,r.col,ss.col,is.col,ip.col))
    combined=plot.top/plot.below+plot_layout(height=c(0.4,1))
    combined[[1]]=combined[[1]]+theme(axis.text.x=element_blank(),
                                      axis.title.x=element_blank())
    plots[[i]]=combined
  }
  return(plots)
}

#Scenario 1
xsevp=0.14
xsevs=0.21
sigmaP=0.8
sigmaS=0.8
delta1=1/(0.5*52)
delta2=2*delta1
delta_vax=10000
alpha=1
epsilon1=0.7
epsilon2=1
plotsAP.scenario.heterogeneity.1=plots.soc.dis.seasonality.vax.heterogeneity(alpha,15,56,0,0,delta1,delta2,
                                                        delta_vax,78,c(0),xsevp,xsevs,epsilon1,epsilon2,
                                                        sigmaP,sigmaS)
                                                        
#Scenario 2                                                                                                                                                                   
xsevp=0.14
xsevs=0.07
sigmaP=0.8
sigmaS=0.8
delta1=1/(2*52)
delta2=2*delta1
delta_vax=10000
alpha=1
epsilon1=0.5
epsilon2=1
plotsAP.scenario.heterogeneity.2=plots.soc.dis.seasonality.vax.heterogeneity(alpha,15,56,0,0,delta1,delta2,
                                                         delta_vax,78,c(0),xsevp,xsevs,epsilon1,epsilon2,
                                                         sigmaP,sigmaS)

#Scenario 3
xsevp=0.14
xsevs=0.21
sigmaP=0.8
sigmaS=0.8
delta1=1/(0.5*52)
delta2=2*delta1
delta_vax=1/(0.25*52)
alpha=1
epsilon1=0.7
epsilon2=1
plotsAP.scenario.heterogeneity.3=plots.soc.dis.seasonality.vax.heterogeneity(1,15,56,0,0,delta1,delta2,
                                                         delta_vax,78,c(0.01),xsevp,xsevs,epsilon1,epsilon2,
                                                         sigmaP,sigmaS)

#Scenario 4                                                                                                                
xsevp=0.14
xsevs=0.07
sigmaP=0.8
sigmaS=0.8
delta1=1/(2*52)
delta2=2*delta1
delta_vax=1/(1*52)
alpha=1
epsilon1=0.5
epsilon2=1
plotsAP.scenario.heterogeneity.4=plots.soc.dis.seasonality.vax.heterogeneity(1,15,56,0,0,delta1,delta2,
                                                         delta_vax,78,c(0.01),xsevp,xsevs,epsilon1,epsilon2,
                                                         sigmaP,sigmaS)

AP.Plots.Heterogeneity=(((plotsAP.scenario.heterogeneity.1[[1]][[1]]/plotsAP.scenario.heterogeneity.1[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
             (plotsAP.scenario.heterogeneity.2[[1]][[1]]/plotsAP.scenario.heterogeneity.2[[1]][[2]]+plot_layout(height=c(0.4,1))))+plot_layout(widths=c(1,0.1,1)))/
  (((plotsAP.scenario.heterogeneity.3[[1]][[1]]/plotsAP.scenario.heterogeneity.3[[1]][[2]]+plot_layout(height=c(0.4,1)))|plot_spacer()|
      (plotsAP.scenario.heterogeneity.4[[1]][[1]]/plotsAP.scenario.heterogeneity.4[[1]][[2]])+plot_layout(height=c(0.4,1)))+plot_layout(widths=c(1,0.1,1)))

pdf("area_plots_heterogeneity_reduced_transmission.pdf",width=10,height=12)
print(AP.Plots.Heterogeneity)
dev.off()
