library(plotly)

endemic.equilibrium=function(parameters){
  beta=parameters[["beta"]]
  epsilon=parameters[["epsilon"]]
  delta=parameters[["delta"]]
  delta_vax=parameters[["delta_vax"]]
  mu=parameters[["mu"]]
  gamma=parameters[["gamma"]]
  nu=parameters[["nu"]]
  R0=beta/(gamma+mu)
  A=(delta*gamma)/(delta+mu)
  B=(delta_vax/(delta_vax+mu))*nu
  C=mu
  D=beta
  E=mu+nu
  G=epsilon*beta
  H=mu+nu-(delta_vax/(delta_vax+mu))*nu
  M=epsilon*A*D-(1/R0)*D*G
  N=C*G+epsilon*A*E-(1/R0)*D*H-(1/R0)*E*G
  P=C*H+epsilon*B*C-(1/R0)*E*H
  root1=(-N+sqrt(N^2-4*M*P))/(2*M)
  root2=(-N-sqrt(N^2-4*M*P))/(2*M)
  if (root1>0  & root2>0){
    stop("TWO POSITIVE ROOTS")
  }
  max.root=max(root1,root2)
  return(max.root)
}


nuVector=seq(from=0,to=0.003,by=0.00001)

valuesTotalInfections=function(epsilon,delta_vax,nuVector){
  mu=1/(365*50)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list=R0.listOrig$hku1
  beta=mean(R0.list)/5
  gamma=1/5
  delta=1/365
  vals.out=rep(0,length(nuVector))
  for (i in 1:length(nuVector)){
    parameters=c(beta=beta,gamma=gamma,epsilon=epsilon,
                 delta_vax=delta_vax,delta=delta,nu=nuVector[i],mu=mu)
    vals.out[i]=endemic.equilibrium(parameters)
  }
  vals.out[vals.out<0]=0
  return(vals.out)
}



epsilon1=0.5
epsilon2=0.7
epsilon3=1
delta_vax1=1/(0.25*365)
delta_vax2=1/(0.5*365)
delta_vax3=1/(1*365)
value11=valuesTotalInfections(epsilon1,delta_vax1,nuVector)
value12=valuesTotalInfections(epsilon1,delta_vax2,nuVector)
value13=valuesTotalInfections(epsilon1,delta_vax3,nuVector)
value21=valuesTotalInfections(epsilon2,delta_vax1,nuVector)
value22=valuesTotalInfections(epsilon2,delta_vax2,nuVector)
value23=valuesTotalInfections(epsilon2,delta_vax3,nuVector)
value31=valuesTotalInfections(epsilon3,delta_vax1,nuVector)
value32=valuesTotalInfections(epsilon3,delta_vax2,nuVector)
value33=valuesTotalInfections(epsilon3,delta_vax3,nuVector)
values=c(value11,value12,value13,value21,value22,value23,
         value31,value32,value33)
e1=rep(epsilon1,length(value11))
e2=rep(epsilon2,length(value21))
e3=rep(epsilon3,length(value31))
d1=rep(1/(delta_vax1*365),length(value11))
d2=rep(1/(delta_vax2*365),length(value21))
d3=rep(1/(delta_vax3*365),length(value31))
epsilon.values=c(e1,e1,e1,e2,e2,e2,e3,e3,e3)
delta.vax.values=c(d1,d2,d3,d1,d2,d3,d1,d2,d3)
nu.values=rep(nuVector,9)
data.plot=data.frame(yy=values,xx=nu.values,colour=as.character(epsilon.values),
                     type=as.character(delta.vax.values))

scene=list(camera=list(eye=list(x=-1.5,y=-1.5,z=-1.5)),
           yaxis=list(title="",
                      titlefont=list(size=20),tickfont=list(size=15),
                      ticksuffix = "   ",automargin=T,
                      tickvals=list(0.25,0.5,1)),
           zaxis=list(title="",titlefont=list(size=20),
                      tickfont=list(size=15),ticksuffix = "   ",automargin=T),
           xaxis=list(title="",titlefont=list(size=20),
                      tickfont=list(size=15),ticksuffix = "   ",automargin=T))
plot.3d=plot_ly(data=data.plot,y=~type,x=~xx,z=~yy,type='scatter3d',
                     mode='lines',color=~type,
                     colors=c("chartreuse4","red","blue"),
                     linetype=~colour,line=list(width=6))%>% 
  layout(scene=scene,showlegend=F)


