library(deSolve)
library(ggplot2)
library(reshape2)
library(plotly)
library(dplyr)
library(tikzDevice)
library(pracma)
library(cowplot)
library(patchwork)


sir_s_.model.seasonality.socdis.vax.hesitancy<-function (t,x,parameters) {
  
  SP1=x[1]
  IP1=x[2]
  R1=x[3]
  SS1=x[4]
  IS1=x[5]
  V1=x[6]
  SP2=x[7]
  IP2=x[8]
  R2=x[9]
  SS2=x[10]
  IS2=x[11]

  mu=parameters[["mu"]]
  delta=parameters[["delta"]]
  delta_vax=parameters[["delta_vax"]]
  gamma=parameters[["gamma"]]
  R0.list=parameters[["R0.list"]]
  alpha=parameters[["alpha"]]
  epsilon=parameters[["epsilon"]]
  beta=R0.list[t]*gamma
  #Social distancing
  R0red=parameters[["R0red"]]
  tred1=parameters[["tred1"]]
  tred2=parameters[["tred2"]]
  #Vaccination
  nu=parameters[["nu"]]
  tvax=parameters[["tvax"]]
  a11=parameters[["a11"]]
  a12=parameters[["a12"]]
  a21=parameters[["a21"]]
  a22=parameters[["a22"]]
  N1=parameters[["N1"]]
  N2=parameters[["N2"]]
  
  if((t > tred1) & (t < tred2)){
    beta = beta * R0red
  }
  
  svax=0
  if(t>tvax){
    svax=1
  }
  
  #Equations
  dSP1dt=mu*N1-(beta*SP1*(a11*(IP1+alpha*IS1)+a12*(IP2+alpha*IS2)))-mu*SP1-svax*nu*SP1
  dIP1dt=(beta*SP1*(a11*(IP1+alpha*IS1)+a12*(IP2+alpha*IS2)))-(gamma+mu)*IP1
  dR1dt=gamma*(IP1+IS1)-(delta+mu)*R1
  dSS1dt=delta*R1-epsilon*(beta*SS1*(a11*(IP1+alpha*IS1)+a12*(IP2+alpha*IS2)))-mu*SS1+delta_vax*V1-svax*nu*SS1
  dIS1dt=epsilon*(beta*SS1*(a11*(IP1+alpha*IS1)+a12*(IP2+alpha*IS2)))-(gamma+mu)*IS1
  dV1dt=svax*nu*(SP1+SS1)-delta_vax*V1-mu*V1
  
  dSP2dt=mu*N2-(beta*SP2*(a22*(IP2+alpha*IS2)+a21*(IP1+alpha*IS1)))-mu*SP2
  dIP2dt=(beta*SP2*(a22*(IP2+alpha*IS2)+a21*(IP1+alpha*IS1)))-(gamma+mu)*IP2
  dR2dt=gamma*(IP2+IS2)-(delta+mu)*R2
  dSS2dt=delta*R2-epsilon*(beta*SS2*(a22*(IP2+alpha*IS2)+a21*(IP1+alpha*IS1)))-mu*SS2
  dIS2dt=epsilon*(beta*SS2*(a22*(IP2+alpha*IS2)+a21*(IP1+alpha*IS1)))-(gamma+mu)*IS2
  
  dxdt <- c(dSP1dt,dIP1dt,dR1dt,dSS1dt,dIS1dt,dV1dt,
            dSP2dt,dIP2dt,dR2dt,dSS2dt,dIS2dt)
  list(dxdt)
}



plots.soc.dis.seasonality.vax.hesitancy=function(alpha,tred1,tred2,
                                                delta,
                                                delta_vax,tvax,
                                                nuVector,
                                                primary.burden,
                                                secondary.burden,
                                                epsilon,
                                                a11,a12,a21,a22,
                                                N2){
  valuesIP=list()
  valuesIS=list()
  valuesR=list()
  valuesSP=list()
  valuesSS=list()
  valuesV=list()
  plots=list()
  N1=1-N2
  mu=1/(50*52)
  gamma=7/5
  R0red=0.6
  times=seq(from=1,to=5*52,by=1)
  R0.listOrig=read.table("NY.csv",header=T,sep=",")
  R0.list.a=R0.listOrig$hku1
  R0.list=rep(R0.list.a,length=length(times))
  I0=1e-9
  sp.col=rgb(0.75,0.75,0.75)
  ip.col=rgb(0.8,0.28,0.23)
  r.col=rgb(0.36,0.33,0.58)
  ss.col=rgb(0.71,0.64,0.83)
  is.col=rgb(1,0.81,0.85)
  v.col=rgb(0.33,0.51,0.21)
  xstart=c(SP1=N1-N1*I0,IP1=N1*I0,R1=0,SS1=0,IS1=0,V1=0,
           SP2=N2-N2*I0,IP2=N2*I0,R2=0,SS2=0,IS2=0) 
  for  (i in  1:length(nuVector)){
    nu=nuVector[i]
    parameters=list(mu=mu,gamma=gamma,alpha=alpha,N2=N2,N1=N1,
                    epsilon=epsilon,
                    delta=delta,
                    R0.list=R0.list,R0red=R0red,tred1=tred1,
                    tred2=tred2,tvax=tvax,
                    nu=nu,
                    a11=a11,a12=a12,
                    a21=a21,a22=a22,
                    delta_vax=delta_vax)
    out=as.data.frame(rk(func=sir_s_.model.seasonality.socdis.vax.hesitancy,
                         y=xstart,times=times,parms=parameters))
    valuesIP[[i]]=out$IP1+out$IP2
    valuesIS[[i]]=out$IS1+out$IS2
    valuesR[[i]]=out$R1+out$R2
    valuesSS[[i]]=out$SS1+out$SS2
    valuesSP[[i]]=out$SP1+out$SP2
    valuesV[[i]]=out$V1
    forAreaPlotIP=valuesIP[[i]]
    forAreaPlotIS=valuesIS[[i]]
    forAreaPlotR=valuesR[[i]]
    forAreaPlotSS=valuesSS[[i]]
    forAreaPlotSP=valuesSP[[i]]
    forAreaPlotV=valuesV[[i]]
    burden.first=primary.burden*forAreaPlotIP
    burden.second=secondary.burden*forAreaPlotIS
    burden=burden.first+burden.second
    data.burden=data.frame(xx=times,yy=burden)
    plot.top=ggplot(data=data.burden)+geom_line(aes(x=xx/52,y=yy))+
      ylab("Severe cases")+xlab("Time (years)")+scale_x_continuous(expand=c(0,0))+
      theme_classic((size=14))
    
    data.AP.IP=data.frame(xx=times,yy=forAreaPlotIP)
    data.AP.IS=data.frame(xx=times,yy=forAreaPlotIS)
    data.AP.R=data.frame(xx=times,yy=forAreaPlotR)
    data.AP.SS=data.frame(xx=times,yy=forAreaPlotSS)
    data.AP.SP=data.frame(xx=times,yy=forAreaPlotSP)
    data.AP.V=data.frame(xx=times,yy=forAreaPlotV)
    
    data.AP.list=list("primary infection"=data.AP.IP,"secondary infection"=data.AP.IS,
                      "fully immune"=data.AP.R,"partially immune"=data.AP.SS,
                      "fully susceptible"=data.AP.SP,
                      "vaccinated"=data.AP.V)
    
    area.plots.timecourses=bind_rows(data.AP.list,.id=c("immunity"))
    area.plots.timecourses$immunity=factor(area.plots.timecourses$immunity,levels=c("vaccinated","fully susceptible","fully immune",
                                                                                    "partially immune","secondary infection","primary infection"))
    plot.below=ggplot(area.plots.timecourses,aes(x=xx/52,y=yy,fill=immunity))+geom_area()+
      theme_classic((size=14))+
      scale_y_continuous(name="Fraction of population",expand=c(0,0))+
      scale_x_continuous(name="Time (years)",expand=c(0,0))+
      scale_fill_manual(name="",values=c(v.col,sp.col,r.col,ss.col,is.col,ip.col))+theme(legend.position="none")
    combined=plot.top/plot.below+plot_layout(height=c(0.4,1))
    combined[[1]]=combined[[1]]+theme(axis.text.x=element_blank(),
                                      axis.title.x=element_blank())
    plots[[i]]=combined
  }
  return(plots)
}


plots.vaccine.hesistancy.scenario.better.immunity.homogeneous=plots.soc.dis.seasonality.vax.hesitancy(alpha=1,tred1=15,tred2=56,
                                                                          delta=1/(2*52),
                                                                          delta_vax=1/(1*52),
                                                                          tvax=78,
                                                                          nuVector=c(0.01),
                                                                          primary.burden=0.14,secondary.burden=0.07,
                                                                          epsilon=0.5,a11=1,a21=1,a12=1,a22=1,N2=0.3)

pdf("AP_refusal_better_immunity_homogeneous_Sept1.pdf",width=3.25,height=4.25)
print(plots.vaccine.hesistancy.scenario.better.immunity.homogeneous)
dev.off()

plots.vaccine.hesistancy.scenario.worse.immunity.homogeneous=plots.soc.dis.seasonality.vax.hesitancy(alpha=1,tred1=15,tred2=56,
                                                                                                    delta=1/(0.5*52),
                                                                                                    delta_vax=1/(0.25*52),
                                                                                                    tvax=78,
                                                                                                    nuVector=c(0.01),
                                                                                                    primary.burden=0.14,secondary.burden=0.21,
                                                                                                    epsilon=0.7,a11=1,a21=1,a12=1,a22=1,N2=0.3)
pdf("AP_refusal_worse_immunity_homogeneous_Sept1.pdf",width=3.25,height=4.25)
print(plots.vaccine.hesistancy.scenario.worse.immunity.homogeneous)
dev.off()


plots.vaccine.hesistancy.scenario.better.immunity.increased.contacts=plots.soc.dis.seasonality.vax.hesitancy(alpha=1,tred1=15,tred2=56,
                                                                                                      delta=1/(2*52),
                                                                                                      delta_vax=1/(1*52),
                                                                                                      tvax=78,
                                                                                                      nuVector=c(0.01),
                                                                                                      primary.burden=0.14,secondary.burden=0.07,
                                                                                                      epsilon=0.5,a11=1,a21=1.25,a12=1.25,a22=1.5,N2=0.3)
pdf("AP_refusal_better_immunity_increased_contacts_Sept1.pdf",width=3.25,height=4.25)
print(plots.vaccine.hesistancy.scenario.better.immunity.increased.contacts)
dev.off()


plots.vaccine.hesistancy.scenario.worse.immunity.increased.contacts=plots.soc.dis.seasonality.vax.hesitancy(alpha=1,tred1=15,tred2=56,
                                                                                                     delta=1/(0.5*52),
                                                                                                     delta_vax=1/(0.25*52),
                                                                                                     tvax=78,
                                                                                                     nuVector=c(0.01),
                                                                                                     primary.burden=0.14,secondary.burden=0.21,
                                                                                                     epsilon=0.7,a11=1,a21=1.25,a12=1.25,a22=1.5,N2=0.3)

pdf("AP_refusal_worse_immunity_increased_contacts_Sept1.pdf",width=3.25,height=4.25)
print(plots.vaccine.hesistancy.scenario.worse.immunity.increased.contacts)
dev.off()


