library(ggplot2)
library(cowplot)
library(tikzDevice)
  
nu.min.vaccine.hesitant.expanded=function(beta,gamma,mu,delta.v,N2,epsilon,c11,c22,c12,c21){
  N1=1-N2
  a1=beta*N1/(gamma+mu)
  a2=mu
  a3=epsilon*delta.v
  a4=mu+delta.v
  A=beta/(gamma+mu)
  a5=((4/A)*(1-c22*N2*A))/((4*c12*c21*N2)-(4*c11*c22*N2)+((4/A)*c11))
  b2=a5
  b1=a5*(a4+a2)-a1*a2-a1*a3
  b0=a1*a4*a2-a5*a2*a4
  root1=(-b1+sqrt(b1^2+4*b0*b2))/(2*b2)
  root2=(-b1-sqrt(b1^2+4*b0*b2))/(2*b2)
  if (root1>0 & root2>0){
    stop("TWO POSITIVE ROOTS")
  }
  nu.min=max(root1,root2)
  if (nu.min<0){
    print("No minimum vaccination rate so that R0<1 (R0>1 always)")
    nu.min=NA
  }
  Smathcal=(N1*((mu/(nu.min+mu))+epsilon*(nu.min/(nu.min+mu))*(delta.v/(delta.v+nu.min+mu))))
  R0check=(A/2)*(sqrt(((c11*Smathcal-c22*N2)^2)+(4*c12*c21*Smathcal*N2))+(c11*Smathcal)+(c22*N2))
  print(R0check)
  return(nu.min)
}

make.plot=function(beta,gamma,mu,epsilon,c11,c22,c12,c21,N2){
  delta.v1=(1/(0.25*365))
  delta.v2=(1/(0.5*365))
  delta.v3=(1/(1*365))
  nu1min.1=rep(NA,length(N2))
  nu1min.2=rep(NA,length(N2))
  nu1min.3=rep(NA,length(N2))
  for (i in 1:length(N2)){
    nu1min.1[i]=nu.min.vaccine.hesitant.expanded(beta=beta,gamma=gamma,mu=mu,
                                        delta.v=delta.v1,N2=N2[i],
                                        epsilon=epsilon,c11=c11,c22=c22,c12=c12,c21=c21)
    nu1min.2[i]=nu.min.vaccine.hesitant.expanded(beta=beta,gamma=gamma,mu=mu,
                                        delta.v=delta.v2,N2=N2[i],
                                        epsilon=epsilon,c11=c11,c22=c22,c12=c12,c21=c21)
    nu1min.3[i]=nu.min.vaccine.hesitant.expanded(beta=beta,gamma=gamma,mu=mu,
                                        delta.v=delta.v3,N2=N2[i],
                                        epsilon=epsilon,c11=c11,c22=c22,c12=c12,c21=c21)
  }
  Nvector=c(N2,N2,N2)
  nuVector=c(nu1min.1,nu1min.2,nu1min.3)
  type=c(rep("0.25",length(nu1min.1)),
         rep("0.5",length(nu1min.2)),rep("1",length(nu1min.3)))
  dataAll=data.frame(N=Nvector,nu=nuVector,type)
  f=ggplot(data=dataAll)+geom_line(aes(x=N,y=nu,linetype=type))+
    ylab("Minimum vaccination rate")+
    xlab("Fraction of population\nthat refuses vaccine")+
    theme(legend.key.width=unit(2,"line"))+
    labs(linetype="Duration of vaccine immunity $1/\\delta_{\\tiny\\textrm{vax}}$ (years)")
  return(f)
}

N2=seq(from=0,to=0.3,length.out=100)
R0.listOrig=read.table("NY.csv",header=T,sep=",")
R0.list.a=R0.listOrig$hku1
beta=mean(R0.list.a)/5
gamma=1/5
mu=1/(50*365)
epsilon=0.5
plot1=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1,c21=1,c22=1)
y.range1=ggplot_build(plot1)$layout$panel_scales_y[[1]]$range$range

epsilon=0.7
plot2=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1,c21=1,c22=1)
y.range2=ggplot_build(plot2)$layout$panel_scales_y[[1]]$range$range


epsilon=0.5
plot3=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1.25,c21=1.25,c22=1.5)
y.range3=ggplot_build(plot3)$layout$panel_scales_y[[1]]$range$range

epsilon=0.7
plot4=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1.25,c21=1.25,c22=1.5)
y.range4=ggplot_build(plot4)$layout$panel_scales_y[[1]]$range$range

epsilon=0.5
plot5=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=0.825,c21=0.825,c22=0.75)
y.range5=ggplot_build(plot5)$layout$panel_scales_y[[1]]$range$range

epsilon=0.7
plot6=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=0.825,c21=0.825,c22=0.75)
y.range6=ggplot_build(plot6)$layout$panel_scales_y[[1]]$range$range

epsilon=1
plot7=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1,c21=1,c22=1)
y.range7=ggplot_build(plot7)$layout$panel_scales_y[[1]]$range$range

epsilon=1
plot8=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=1.25,c21=1.25,c22=1.5)
y.range8=ggplot_build(plot8)$layout$panel_scales_y[[1]]$range$range

epsilon=1
plot9=make.plot(beta=beta,gamma=gamma,mu=mu,epsilon=epsilon,N2=N2,
                c11=1,c12=0.825,c21=0.825,c22=0.75)
y.range9=ggplot_build(plot9)$layout$panel_scales_y[[1]]$range$range



max.y=max(y.range1[2],y.range2[2],y.range3[2],
          y.range4[2],y.range5[2],y.range6[2],
          y.range7[2],y.range8[2],y.range9[2])
min.y=min(y.range1[1],y.range2[1],y.range3[1],
          y.range4[1],y.range5[1],y.range6[1],
          y.range7[1],y.range8[1],y.range9[1])

plot.new1=plot1+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.x=element_blank())

plot.new2=plot2+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.x=element_blank(),axis.text.y=element_blank())

plot.new3=plot3+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.x=element_blank())

plot.new4=plot4+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.x=element_blank(),axis.text.y=element_blank())

plot.new5=plot5+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank())

plot.new6=plot6+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.y=element_blank())

plot.new7=plot7+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.x=element_blank(),axis.text.y=element_blank())

plot.new8=plot8+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
          axis.text.x=element_blank(),axis.text.y=element_blank())

plot.new9=plot9+scale_y_continuous(trans="log10",limits=c(min.y,max.y))+
  theme(legend.position="none")+
  theme(axis.title.x=element_blank(),axis.title.y=element_blank(),
        axis.text.y=element_blank())

plots.without.legend=plot_grid(plot.new1,plot.new2,plot.new7,
                               plot.new3,plot.new4,plot.new8,
                               plot.new5,plot.new6,plot.new9,
                               nrow=3,align="hv",labels=NULL)
legendAll=get_legend(plot1+theme(legend.position="top")+
                       theme(legend.title=element_text(size=12),
                             legend.text=element_text(size=12.5))+
                       theme(legend.box.margin=margin(0,0,0,0)))
empty=ggdraw()
plots.with.legend=plot_grid(plots.without.legend,empty,legendAll,nrow=3,rel_heights=c(4,0.15,0.6))

#for legend
tikz("legend_Aug27.tex",standAlone=T,width=8,height=6)
print(plots.with.legend)
dev.off()

tikz("vaccine_refusal_extended_Sept1.tex",standAlone=T,width=6,height=4)
print(plots.without.legend)
dev.off()


c22=seq(from=0,to=4,by=0.01)
N2calculation=(1/c22)*((gamma+mu)/beta)
N2values=N2calculation
N2values[N2values>1]=1
dataN2min=data.frame(N2=N2values,c22)
minN2plot=ggplot(data=dataN2min)+geom_line(aes(x=c22,y=N2))+
  scale_x_continuous(expand=c(0,0),breaks=c(0,1,2,3,4))+
  scale_y_continuous(expand=c(0,0),breaks=c(0,0.5,1),limits=c(0,1.01))+
  ylab("Fraction refusing\nvaccination")+xlab("Relative contact rate")+
  annotate("text",label="$\\mathcal{R}_0<1$ cannot be\nachieved by\nvaccination alone",
            x=2.5,y=0.7,size=4.5)
tikz("minN2_c22_Sept1.tex",standAlone=T,width=3,height=2.25)
print(minN2plot)
dev.off()


